import grok
import re
from zope.component import getUtility, getMultiAdapter
from zope.interface import Interface

from waeup.interfaces import (IUniversity, IStudentContainer, IHostelContainer,
                              IFacultyContainer)
from waeup.viewlets import MainArea, LeftSidebar, Index, Manage

class University(grok.Application, grok.Container):
    """A university.
    """
    grok.implements(IUniversity)

    def __init__(self, name=u'Sample University', **kw):
        super(University, self).__init__(**kw)
        self.name = name
        self.setup()

    def setup(self):
        self["students"] = getUtility(IStudentContainer)
        self["hostels"] = getUtility(IHostelContainer)
        self["faculties"] = getUtility(IFacultyContainer)

        
        
class ManageForm(grok.EditForm):
    """Manage the basic properties of a `University` instance.
    """

    form_fields = grok.AutoFields(IUniversity)

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class ManageUniversity(grok.Viewlet):
    """A viewlet that renders the `ManageForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IUniversity)
    grok.view(Manage)
    grok.require('waeup.manageUniversity')
    
    def update(self):
        self.form = getMultiAdapter((self.context, self.request),
                                    name='manageform')
        self.form.update_form()
        if self.request.method == 'POST':
            #app = get_application(self.context)
            #self.__parent__.redirect(self.__parent__.url(obj=app))
            pass

    def render(self):
        result = self.form.render()
        try:
            # strip all except the form part...
            result = re.match('^.+\(<form[^\>]+>.*</form>).+$', result,
                              re.DOTALL).groups()[0]
        except:
            # except there is no such part...
            pass
        return result
        return self.form.render()

    
    
class Content(grok.Viewlet):
    grok.viewletmanager(MainArea)
    grok.context(IUniversity)
    grok.view(Index)

class ManageLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(Index)
    grok.order(5)
    # This is so cool! This link is only displayed, when the user is
    # allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="manage">Settings</a></div>'
